/*
 * Smart GWT (GWT for SmartClient)
 * Copyright 2008 and beyond, Isomorphic Software, Inc.
 *
 * Smart GWT is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License version 3
 * as published by the Free Software Foundation.  Smart GWT is also
 * available under typical commercial license terms - see
 * http://smartclient.com/license
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 */
/* sgwtgen */
 
package com.smartgwt.client.data;


import com.smartgwt.client.event.*;
import com.smartgwt.client.core.*;
import com.smartgwt.client.types.*;
import com.smartgwt.client.data.*;
import com.smartgwt.client.data.events.*;
import com.smartgwt.client.rpc.*;
import com.smartgwt.client.callbacks.*;
import com.smartgwt.client.tools.*;
import com.smartgwt.client.bean.*;
import com.smartgwt.client.widgets.*;
import com.smartgwt.client.widgets.events.*;
import com.smartgwt.client.widgets.form.*;
import com.smartgwt.client.widgets.form.validator.*;
import com.smartgwt.client.widgets.form.fields.*;
import com.smartgwt.client.widgets.tile.*;
import com.smartgwt.client.widgets.tile.events.*;
import com.smartgwt.client.widgets.grid.*;
import com.smartgwt.client.widgets.grid.events.*;
import com.smartgwt.client.widgets.chart.*;
import com.smartgwt.client.widgets.layout.*;
import com.smartgwt.client.widgets.layout.events.*;
import com.smartgwt.client.widgets.menu.*;
import com.smartgwt.client.widgets.rte.*;
import com.smartgwt.client.widgets.rte.events.*;
import com.smartgwt.client.widgets.tab.*;
import com.smartgwt.client.widgets.toolbar.*;
import com.smartgwt.client.widgets.tree.*;
import com.smartgwt.client.widgets.tree.events.*;
import com.smartgwt.client.widgets.viewer.*;
import com.smartgwt.client.widgets.calendar.*;
import com.smartgwt.client.widgets.calendar.events.*;
import com.smartgwt.client.widgets.cube.*;
import com.smartgwt.client.widgets.drawing.*;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import com.google.gwt.user.client.ui.Widget;
import com.google.gwt.core.client.JavaScriptObject;
import com.google.gwt.user.client.Element;
import com.smartgwt.client.util.*;
import com.smartgwt.client.util.workflow.*;
import com.google.gwt.event.shared.*;
import com.google.gwt.event.shared.HasHandlers;

/**
 * A special kind of {@link com.smartgwt.client.data.DataSource#getClientOnly client-only DataSource} that can be
 * configured with {@link com.smartgwt.client.data.MockDataSource#getMockData "mock data"} - a simple text format for table
 * or tree data. <p> MockDataSources are produces by the Reify Mockup Importer when starting from mockup formats that use
 * the mock data format.  The docs for the  {@link com.smartgwt.client.docs.BmmlImporter Reify Mockup Importer} explain
 * various steps for converting a <code>MockupDataSource</code> to a real DataSource. <p> <code>MockupDataSource</code> is
 * primarily intended as a temporary form of DataSource used during the process of converting a mockup into a real
 * application.  Generally, if creating a client-only DataSource in  Java, there is no reason to use the mock data format,
 * as the mock data is not especially readable when written as a String literal.  The mock data format <i>can</i> be a
 * slightly more compact and readable as compared to declaring {@link com.smartgwt.client.data.DataSource#getTestData
 * testData} in XML.
 */
@BeanFactory.FrameworkClass
@BeanFactory.ScClassName("MockDataSource")
public class MockDataSource extends DataSource {

    public static MockDataSource getOrCreateRef(JavaScriptObject jsObj) {
        if(jsObj == null) return null;
        BaseClass obj = BaseClass.getRef(jsObj);
        if(obj != null) {
            return (MockDataSource) obj;
        } else {
            return new MockDataSource(jsObj);
        }
    }


    public MockDataSource(){
        scClassName = "MockDataSource";
    }

    public MockDataSource(JavaScriptObject jsObj){
        scClassName = "MockDataSource";
        setJavaScriptObject(jsObj);
    }

    public native JavaScriptObject create()/*-{
        var config = this.@com.smartgwt.client.core.BaseClass::getConfig()();
        var scClassName = this.@com.smartgwt.client.core.BaseClass::scClassName;
        return $wnd.isc[scClassName].create(config);
    }-*/;

    // ********************* Properties / Attributes ***********************

    /**
     * Data intended for a {@link com.smartgwt.client.widgets.grid.ListGrid} or {@link
     * com.smartgwt.client.widgets.tree.TreeGrid}, expressed in a simple text format popularized by mockup tools such as <a
     * href='http://balsamiq.com' target='_blank'>http://balsamiq.com</a> and now commonly supported in a variety of mockup
     * tools. <p> Balsamiq publishes documentation of the grid format  <a
     * href='http://support.balsamiq.com/customer/portal/articles/110188-working-with-data-grids-tables'
     * target='_blank'>here</a>, with a simple example of using tree-specific formatting <a
     * href='https://support.mybalsamiq.com/projects/uilibrary/Tree%20Pane' target='_blank'>here</a>.
     *
     * @param mockData  Default value is "md"
     * @throws IllegalStateException this property cannot be changed after the underlying component has been created
     */
    public void setMockData(String mockData)  throws IllegalStateException {
        setAttribute("mockData", mockData, false);
    }

    /**
     * Data intended for a {@link com.smartgwt.client.widgets.grid.ListGrid} or {@link
     * com.smartgwt.client.widgets.tree.TreeGrid}, expressed in a simple text format popularized by mockup tools such as <a
     * href='http://balsamiq.com' target='_blank'>http://balsamiq.com</a> and now commonly supported in a variety of mockup
     * tools. <p> Balsamiq publishes documentation of the grid format  <a
     * href='http://support.balsamiq.com/customer/portal/articles/110188-working-with-data-grids-tables'
     * target='_blank'>here</a>, with a simple example of using tree-specific formatting <a
     * href='https://support.mybalsamiq.com/projects/uilibrary/Tree%20Pane' target='_blank'>here</a>.
     *
     * @return String
     */
    public String getMockData()  {
        return getAttributeAsString("mockData");
    }
    

    /**
     * Whether {@link com.smartgwt.client.data.MockDataSource#getMockData mockData} is in the "grid" or "tree" format.  See
     * {@link com.smartgwt.client.types.MockDataType}.
     *
     * @param mockDataType  Default value is "grid"
     * @throws IllegalStateException this property cannot be changed after the underlying component has been created
     */
    public void setMockDataType(MockDataType mockDataType)  throws IllegalStateException {
        setAttribute("mockDataType", mockDataType == null ? null : mockDataType.getValue(), false);
    }

    /**
     * Whether {@link com.smartgwt.client.data.MockDataSource#getMockData mockData} is in the "grid" or "tree" format.  See
     * {@link com.smartgwt.client.types.MockDataType}.
     *
     * @return MockDataType
     */
    public MockDataType getMockDataType()  {
        return EnumUtil.getEnum(MockDataType.values(), getAttribute("mockDataType"));
    }
    

    // ********************* Methods ***********************

    // ********************* Static Methods ***********************

    // ***********************************************************

}


